/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.autolayout;

import java.util.ArrayList;

/**
 * 节点
 * 
 * @author luoguo
 * 
 */
public class Node {
	public static final int POSITION_NONE = 0;// 独立
	public static final int POSITION_MAIN = 1;// 主线
	public static final int POSITION_SUB = 2;// 分支
	public static final int POSITION_COMMON = 3;// 普通

	public static final int TYPE_NONE = 1;// 独立
	public static final int TYPE_START = 1;// 开始
	public static final int TYPE_END = 2;// 结束
	public static final int TYPE_COMMON = 3;// 普通

	int startLinkCount = 0;// 开始连接数
	int endLinkCount = 0;// 结束连接数
	String caption;// 标题
	int type = TYPE_COMMON;
	int mainNode = POSITION_NONE;

	public int getMainNode() {
		return mainNode;
	}

	public void setMainNode(int mainNode) {
		this.mainNode = mainNode;
	}

	Point point = new Point(0, 0);
	ArrayList<Link> linkList = new ArrayList<Link>();

	/**
	 * 构建开始结点
	 * 
	 * @return
	 */
	public static Node getStartNode() {
		return new Node("Start", TYPE_START);
	}

	/**
	 * 返回结束节点
	 * 
	 * @return
	 */
	public static Node getEndNode() {
		return new Node("End", TYPE_END);
	}

	public Point getPoint() {
		return point;
	}

	public void setPoint(Point point) {
		this.point = point;
	}

	private Node(String caption, int type) {
		this.caption = caption;
		this.type = type;
	}

	public Node(String caption) {
		this.caption = caption;
		this.type = TYPE_COMMON;
	}

	public void linkNode(Node dest) throws Exception {
		new Link(this, dest);
		startLinkCount++;
	}

	public void linkByNode(Node source) throws Exception {
		new Link(source, this);
		endLinkCount++;
	}

	public Node getNextNode() throws Exception {
		for (Link link : linkList) {
			if (link.start == this) {
				return link.end;
			}
		}
		throw new Exception("There is no next node");
	}

	public Node getPreviousNode() throws Exception {
		for (Link link : linkList) {
			if (link.end == this) {
				return link.start;
			}
		}
		throw new Exception("There is no next node");
	}

	public String getCaption() {
		return caption;
	}

	public void setCaption(String caption) {
		this.caption = caption;
	}

	public int getType() {
		return type;
	}

	public void setType(int type) {
		this.type = type;
	}

	public ArrayList<Link> getLinkList() {
		return linkList;
	}

	public void setLinkList(ArrayList<Link> linkList) {
		this.linkList = linkList;
	}

	public void print() {
		System.out.printf("%s(%d,%d)\n", caption, point.x, point.y);
	}

}
